Attribute VB_Name = "AssemblyTraverse"
Option Compare Binary
' --------------------------------------------------------------------03/09/2007
' ModelProperties.swp               Written by Leonard Kikstra,
'                                   Copyright 2004-2007, Leonard Kikstra
'                                   Downloaded from Lenny's SolidWorks Resources
'                                        at http://www.lennyworks.com/solidworks
' ------------------------------------------------------------------------------
' The 'TraverseComponent' in this module is separated by assembly level to
' maintain contact with 'ModelDoc2' of ' parent assembly and to provide the
' capability of retrieving the 'ReferencedConfiguration' of 'Child' components.
' Many portions of the routines are similar, but no subroutines were created
' with the exception of the 'FormBOM.MakeTempBOM' routine used to create the
' temporary BOM output.
'------------------------------------------------------------------------------
Dim RootComponent As Object
Dim TempName As String
Dim ReadOnly As String

' Recursive routine to traverse all the children of a component
Function TraverseComponent(Component2, mode)
  Dim i As Integer
  Dim Children   As Variant
  Dim Child      As Object
  Dim ChildCount As Integer
  Dim ModDoc     As Object      ' ModelDoc2 of child component
  Dim ModConfig  As Object      ' Configuration object of current component
  Dim ConfigName As String      ' Configuration name of current component
  Dim FileTyp    As Integer
  Dim FileERR As Long
  If Not Component2.IsSuppressed() Then
    Set ModDoc = Component2.GetModelDoc()
    If Not ModDoc Is Nothing Then
      ConfigName = Component2.ReferencedConfiguration
      Set ModDocConfig = ModDoc.GetActiveConfiguration
      If Component2.ReferencedConfiguration <> ModDocConfig.Name Then
        ModDoc.ShowConfiguration2 ConfigName        ' Show ref'd config
      End If                                        ' only if not active
      FileTyp = ModDoc.GetType                      ' Get document type
      If FileTyp <> swDocAssembly Then              ' Not an assembly
        Select Case mode
               Case 0           ' Change component color
                    SetColor ModDoc
               Case 1           ' Change component props
                    DynamicProp ModDoc
               Case 2           ' Remove Comp & Face Colors
                    Component2.RemoveMaterialProperty
                    RemoveFaceColors Component2
               Case Else
        End Select
      End If        ' End Document Type
    End If          ' ModDoc is Nothing
    Children = Component2.GetChildren               ' Get list / children
    If TypeName(Children) = "Object()" Then
      ChildCount = UBound(Children) + 1             ' Get Child Count
      For i = 0 To (ChildCount - 1)                 ' Each child in sub
        Set Child = Children(i)                     ' Get child comp obj
        TraverseComponent Child, mode               ' Traverse child's comp
      Next i
    End If          ' Valid child list
  End If            ' Component suppressed
End Function

Sub TraverseFeatures(swFeat)
  swFeat.RemoveMaterialProperty                     ' Remove properties
  Set swSubAFeat = swFeat.GetFirstSubFeature        ' Get first feature
  While Not swSubAFeat Is Nothing                   ' Feature is valid
    swSubAFeat.RemoveMaterialProperty               ' Remove properties
    Set swSubBFeat = swSubAFeat.GetFirstSubFeature
    While Not swSubBFeat Is Nothing
      swSubBFeat.RemoveMaterialProperty
      Set swSubCFeat = swSubBFeat.GetFirstSubFeature
      While Not swSubCFeat Is Nothing
        swSubCFeat.RemoveMaterialProperty
        Set swSubCFeat = swSubCFeat.GetNextSubFeature
      Wend
      Set swSubBFeat = swSubBFeat.GetNextSubFeature
    Wend
    Set swSubAFeat = swSubAFeat.GetNextSubFeature
  Wend
End Sub

Sub RemoveFaceColors(ThisModel)
  Dim FileERR As Long
  Dim FileWARN As Long
  Set CompModDoc = ThisModel.GetModelDoc()          ' Get ModelDoc - ThisModel
  CompName = CompModDoc.GetPathName                 ' and make model activate
  Set LoadedModDoc = swApp.ActivateDoc2(CompName, True, FileERR)
  If Not LoadedModDoc Is Nothing Then               ' Ensure model is loaded
    Set swBody = LoadedModDoc.Body
    Set swFace = swBody.GetFirstFace
    While Not swFace Is Nothing
      swFace.RemoveMaterialProperty                 ' Remove face properties
      Set swFeat = swFace.GetFeature                ' Traverse faces & features
      While Not swFeat Is Nothing
        TraverseFeatures swFeat
        Set swFeat = swFeat.GetNextFeature
      Wend
      Set swFace = swFace.GetNextFace
    Wend
  End If
  Set LoadedModDoc = Nothing                        ' Clear LoadedModDoc
  swApp.QuitDoc (CompName)                          ' Quit model
End Sub

Private Sub SetColor(Model As Object)
  Select Case FormModelProperties.ComboBoxApply.ListIndex
         Case 0     ' Set Shaded color only
                CompShadedColor Model
         Case 1     ' Set Wireframe color only
                CompWireframeColor Model
         Case 2     ' Set Shaded & Wireframe colors
                CompShadedColor Model
                CompWireframeColor Model
         Case Else
  End Select
  ModelDoc2.ForceRebuild3 True
  ModelDoc2.GraphicsRedraw2
End Sub

Sub CompShadedColor(Model)
  MyColor = Right$("000000" + Hex(SelectedColor), 6)
  RetVar = Model.MaterialPropertyValues             ' Read props from model
    RetVar(0) = Val("&H" + Mid(MyColor, 5, 2)) / 255            ' Red Value
    RetVar(1) = Val("&H" + Mid(MyColor, 3, 2)) / 255            ' Green Value
    RetVar(2) = Val("&H" + Mid(MyColor, 1, 2)) / 255            ' Blue Value
  ' RetVar(3)                                                   ' Ambient
  ' RetVar(4)                                                   ' Diffuse
  ' RetVar(5)                                                   ' Specularity
  ' RetVar(6)                                                   ' Shininess
  ' RetVar(7)                                                   ' Transparency
  ' RetVar(8)                                                   ' Emission
  Model.MaterialPropertyValues = RetVar             ' Write props to model
End Sub

Sub CompWireframeColor(Model)
    Model.SetUserPreferenceIntegerValue _
        swDocumentColorWireFrame, SelectedColor
    Model.SetUserPreferenceIntegerValue _
        swDocumentColorHidden, SelectedColor
End Sub

Sub DynamicProp(Model)
  ' Is color value set properly?  If yes, then apply
  ' Set material color properties and lights
  RetVar = Model.MaterialPropertyValues             ' Read props from model
  ' RetVar(0) =                                                  ' Red Value
  ' RetVar(1) =                                                  ' Green Value
  ' RetVar(2) =                                                  ' Blue Value
    RetVar(3) = 1#                                               ' Ambient
    RetVar(4) = 1#                                               ' Diffuse
    RetVar(5) = FormModelProperties.SpinSpec.Value / Resolution  ' Specularity
    RetVar(6) = 1#                                               ' Shininess
    RetVar(7) = FormModelProperties.SpinTrans.Value / Resolution ' Transparency
    RetVar(8) = 0.2                                              ' Emission
  Model.MaterialPropertyValues = RetVar             ' Write props to model
End Sub

